# THIS IS DEPRECATED.  USE boinc_submit.py INSTEAD

# This file is part of BOINC.
# http://boinc.berkeley.edu
# Copyright (C) 2020 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.


# Python bindings of the remote job submission and file management APIs
# See https://github.com/BOINC/boinc/wiki/RemoteJobs#Pythonbinding

import urllib
import urllib2
import copy
import xml.etree.ElementTree as ET
import requests
    # you'll need to "pip install requests"
import hashlib

# describes an input file
#
class FILE_DESC:
    def __init__(self):
        return
    def to_xml(self):
        xml = ('<input_file>\n'
        '<mode>%s</mode>\n'
        ) %(self.mode)
        if self.mode == 'remote':
            xml += ('<url>%s</url>\n'
            '<nbytes>%f</nbytes>\n'
            '<md5>%s</md5>\n'
            ) %(self.url, self.nbytes, self.md5)
        else:
            xml += '<source>%s</source>\n' %(self.source)
        xml += '</input_file>\n'
        return xml

# describes a job
#
class JOB_DESC:
    def __init__(self):
        return
    def to_xml(self):
        xml = '<job>\n'
        if hasattr(self, 'name'):
            xml += '<name>%s</name>\n'%self.name
        if hasattr(self, 'rsc_fpops_est'):
            xml += '<rsc_fpops_est>%f</rsc_fpops_est>\n'%self.rsc_fpops_est
        if hasattr(self, 'command_line'):
            xml += '<command_line>%s</command_line>\n'%self.command_line
        if hasattr(self, 'input_template'):
            xml += '%s\n'%self.input_template
        if hasattr(self, 'output_template'):
            xml += '%s\n'%self.output_template
        if hasattr(self, 'priority'):
            xml += '<priority>%d</priority>\n'%(self.priority)
        if hasattr(self, 'files'):
            for file in self.files:
                xml += file.to_xml()
        xml += '</job>\n'
        return xml

# describes a batch for submit() or estimate()
#
class BATCH_DESC:
    def __init__(self):
        return

    def to_xml(self, op):
        xml = ('<%s>\n'
            '<authenticator>%s</authenticator>\n'
            '<batch>\n'
            '<app_name>%s</app_name>\n'
        ) %(op, self.authenticator, self.app_name)

        if hasattr(self, 'batch_id'):
            xml += '<batch_id>%s</batch_id>\n'%(self.batch_id)
        elif hasattr(self, 'batch_name'):
            xml += '<batch_name>%s</batch_name>\n'%(self.batch_name)

        if hasattr(self, 'app_version_num'):
            xml += '<app_version_num>%d</app_version_num>\n'%(self.app_version_num)

        if hasattr(self, 'allocation_priority'):
            if self.allocation_priority:
                xml += '<allocation_priority/>\n'
        if hasattr(self, 'priority'):
            xml += '<priority>%d</priority>\n'%(self.priority)
        for job in self.jobs:
            xml += job.to_xml()
        xml += '</batch>\n</%s>\n' %(op)
        return xml

class CREATE_BATCH_REQ:
    def __init__(self):
        return
    def to_xml(self):
        xml = ('<create_batch>\n'
        '<authenticator>%s</authenticator>\n'
        '<app_name>%s</app_name>\n'
        '<batch_name>%s</batch_name>\n'
        '<expire_time>%f</expire_time>\n'
        '</create_batch>\n') %(self.authenticator, self.app_name, self.batch_name, self.expire_time)
        return xml

# a generic request
#
class REQUEST:
    def __init__(self):
        return

rpc_timeout = 0

def do_http_post(req, project_url, handler='submit_rpc_handler.php'):
    #print(req)
    url = project_url + handler
    params = urllib.urlencode({'request': req})
    if rpc_timeout>0:
        f = urllib2.urlopen(url, params, rpc_timeout)
    else:
        f = urllib2.urlopen(url, params)

    reply = f.read()
    #print("REPLY:", reply)
    return ET.fromstring(reply)

########### API FUNCTIONS START HERE ###############

def set_timeout(x):
    global rpc_timeout
    rpc_timeout = x

def abort_batch(req):
    req_xml = ('<abort_batch>\n'
    '<authenticator>%s</authenticator>\n'
    '<batch_id>%s</batch_id>\n'
    '</abort_batch>\n'
    ) %(req.authenticator, req.batch_id)
    return do_http_post(req_xml, req.project)

def abort_jobs(req):
    req_xml = ('<abort_jobs>\n'
    '<authenticator>%s</authenticator>\n'
    ) %(req.authenticator)
    for job in req.jobs:
        req_xml += '<job_name>%s</job_name>\n'%(job)
    req_xml += '</abort_jobs>\n'
    return do_http_post(req_xml, req.project)

# req is a CREATE_BATCH_REQ
#
def create_batch(req):
    return do_http_post(req.to_xml(), req.project)

def estimate_batch(req):
    return do_http_post(req.to_xml('estimate_batch'), req.project)

def query_batch(req):
    req_xml = ('<query_batch>\n'
    '<authenticator>%s</authenticator>\n'
    '<batch_id>%s</batch_id>\n'
    '<get_cpu_time>%d</get_cpu_time>\n'
    '<get_job_details>%d</get_job_details>\n'
    '</query_batch>\n'
    ) %(req.authenticator, req.batch_id, 1 if req.get_cpu_time else 0, 1 if req.get_job_details else 0)
    return do_http_post(req_xml, req.project)

def query_batches(req):
    req_xml = ('<query_batches>\n'
    '<authenticator>%s</authenticator>\n'
    '<get_cpu_time>%d</get_cpu_time>\n'
    '</query_batches>\n'
    ) %(req.authenticator, 1 if req.get_cpu_time else 0)
    return do_http_post(req_xml, req.project)

def query_completed_job(req):
    req_xml = ('<query_completed_job>\n'
    '<authenticator>%s</authenticator>\n'
    '<job_name>%s</job_name>\n'
    '</query_completed_job>\n'
    ) %(req.authenticator, req.job_name)
    return do_http_post(req_xml, req.project)

def query_job(req):
    req_xml = ('<query_job>\n'
    '<authenticator>%s</authenticator>\n'
    '<job_id>%s</job_id>\n'
    '</query_job>\n'
    ) %(req.authenticator, req.job_id)
    return do_http_post(req_xml, req.project)

def get_output_file(req):
    auth_str = hashlib.md5(req.authenticator+req.instance_name).hexdigest()
    name = req.instance_name
    file_num = req.file_num
    return req.project+"/get_output.php?cmd=result_file&result_name=%s&file_num=%s&auth_str=%s"%(name, file_num, auth_str)

def get_output_files(req):
    auth_str = hashlib.md5(req.authenticator+str(req.batch_id)).hexdigest()
    return req.project+"/get_output.php?cmd=batch_files&batch_id=%s&auth_str=%s"%(req.batch_id, auth_str)

def retire_batch(req):
    req_xml = ('<retire_batch>\n'
    '<authenticator>%s</authenticator>\n'
    '<batch_id>%s</batch_id>\n'
    '</retire_batch>\n'
    ) %(req.authenticator, req.batch_id)
    return do_http_post(req_xml, req.project)

def submit_batch(req):
    return do_http_post(req.to_xml('submit_batch'), req.project)

# see if reply is error.
# if so print the message and return True
#
def check_error(response):
    if response.find('error') is not None:
         print('BOINC server error: ', response.find('error').find('error_msg').text)
         return True

############ FILE MANAGEMENT API ##############

class QUERY_FILES_REQ:
    def __init__(self):
        return

    def to_xml(self):
        xml = ('<query_files>\n'
        '<authenticator>%s</authenticator>\n'
        '<batch_id>%d</batch_id>\n') %(self.authenticator, self.batch_id)
        for name in self.boinc_names:
            xml += '<phys_name>%s</phys_name>\n' %(name)
        xml += '</query_files>\n'
        return xml

class UPLOAD_FILES_REQ:
    def __init__(self):
        return

    def to_xml(self):
        xml = ('<upload_files>\n'
        '<authenticator>%s</authenticator>\n'
        '<batch_id>%d</batch_id>\n') %(self.authenticator, self.batch_id)
        for name in self.boinc_names:
            xml += '<phys_name>%s</phys_name>\n' %(name)
        xml += '</upload_files>\n'
        return xml

def query_files(query_req):
    reply = do_http_post(query_req.to_xml(), query_req.project, 'job_file.php')
    return reply

# This actually does two RPCs:
# query_files() to find what files aren't already on server
# upload_files() to upload them
#
def upload_files(upload_files_req):
    query_req = QUERY_FILES_REQ()
    query_req.authenticator = upload_files_req.authenticator
    query_req.batch_id = upload_files_req.batch_id
    query_req.boinc_names = upload_files_req.boinc_names
    query_req_xml = query_req.to_xml()
    reply = do_http_post(query_req_xml, upload_files_req.project, 'job_file.php')
    if reply[0].tag == 'error':
        return reply

    absent = reply.find('absent_files').findall('file')
    #print('query files succeeded; ',len(absent), ' files need upload')
    boinc_names = []
    local_names = []
    for n in absent:
        ind = int(n.text)
        boinc_names.append(upload_files_req.boinc_names[ind])
        local_names.append(upload_files_req.local_names[ind])
    upload_files_req.boinc_names = boinc_names
    upload_files_req.local_names = local_names

    # make a description of upload files for "requests"
    #
    files = []
    for i in range(len(boinc_names)):
        bn = boinc_names[i]
        ln = local_names[i]
        upload_name = 'file_%d'%(i)
        files.append((upload_name, (bn, open(ln, 'rb'), 'application/octet-stream')))

    url = upload_files_req.project + '/job_file.php'
    req_xml = upload_files_req.to_xml()
    #print(req_xml)
    req = {'request': req_xml}
    reply = requests.post(url, data=req, files=files)
    #print("reply text: ", reply.text)
    return ET.fromstring(reply.text)

# returns an XML object with various job counts
#   results_ready_to_send
#   results_in_progress
#   results_need_file_delete
#   wus_need_validate
#   wus_need_assimilate
#   wus_need_file_delete
# see tools/submit_api_test.py
#
def get_job_counts(req):
    return do_http_post('', req.project, 'server_status.php?counts=1')



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys

def main():
    if len(sys.argv) != 4:
        print("Usage: msi_validation.py <msival2_path> <boinc_msi_path> <darice_cub_path>")
        sys.exit(1)

    msival2_path = sys.argv[1]
    if not os.path.exists(msival2_path):
        print(f"'{msival2_path}' not found")
        sys.exit(1)

    msi_path = sys.argv[2]
    if not os.path.exists(msi_path):
        print(f"'{msi_path}' not found")
        sys.exit(1)

    cub_path = sys.argv[3]
    if not os.path.exists(cub_path):
        print(f"'{cub_path}' not found")
        sys.exit(1)

    ignore_list = [
        "ICE          Type       Description",
        "ICE07        WARNING   '_BOINCScreensaver_LiberationSans_Regular.ttf' is a Font and must be installed to the FontsFolder. Current Install Directory: 'INSTALLDIR'",
        "ICE30        WARNING   The target file 'BOINC.EXE|boinc.exe' might be installed in '[ProgramFiles64Folder]\\BOINC\\' by two different conditionalized components on an SFN system: 'BOINCServiceConfig' and '_BOINC'. If the conditions are not mutually exclusive, this will break the component reference counting system.",
        "ICE30        WARNING   The target file 'BOINC.EXE|boinc.exe' might be installed in '[ProgramFiles64Folder]\\BOINC\\' by two different conditionalized components on an LFN system: 'BOINCServiceConfig' and '_BOINC'. If the conditions are not mutually exclusive, this will break the component reference counting system.",
        "ICE30        WARNING   The target file 'BOINC.EXE|boinc.exe' might be installed in '[ProgramFiles64Folder]\\BOINC\\' by two different conditionalized components on an SFN system: '_BOINC' and 'BOINCServiceConfig'. If the conditions are not mutually exclusive, this will break the component reference counting system.",
        "ICE30        WARNING   The target file 'BOINC.EXE|boinc.exe' might be installed in '[ProgramFiles64Folder]\\BOINC\\' by two different conditionalized components on an LFN system: '_BOINC' and 'BOINCServiceConfig'. If the conditions are not mutually exclusive, this will break the component reference counting system.",
        "ICE43        ERROR     Component _BOINCManagerStartMenu has non-advertised shortcuts. It should use a registry key under HKCU as its KeyPath, not a file.",
        "ICE57        ERROR     Component '_ScreensaverEnableNT' has both per-user and per-machine data with a per-machine KeyPath.",
        "ICE57        ERROR     Component '_BOINCManagerStartup' has both per-user and per-machine data with a per-machine KeyPath.",
        "ICE57        ERROR     Component '_BOINCManagerStartMenu' has both per-user and per-machine data with a per-machine KeyPath.",
        "ICE61        WARNING   This product should remove only older versions of itself. The Maximum version is not less than the current product.",
    ]
    output = os.popen(f'"{msival2_path}" "{msi_path}" "{cub_path}" -f').read()
    error_found = False
    for line in output.splitlines():
        if line == '' or any(ignore in line for ignore in ignore_list):
            continue
        error_found = True
        print(line)

    if error_found:
        print("Validation failed")
        sys.exit(1)

    print("Validation succeeded")
    sys.exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
import sys

def get_vcpkg_ports():
    vcpkg_ports = []
    for root, _, files in os.walk('3rdParty/vcpkg_ports/ports'):
        for file in files:
            if file == 'vcpkg.json':
                vcpkg_ports.append(os.path.join(root, file))
    return vcpkg_ports

def get_port_info(port):
    with open(port, 'r') as f:
        data = json.load(f)

    name = data.get('name')
    version = data.get('version')
    port_version = data.get('port-version', 0)

    return name, version, port_version

def parse_ports(file_path):
    with open(file_path, 'r') as f:
        data = json.load(f)

    ports_info = []
    for port, details in data.get('default', {}).items():
        baseline = details.get('baseline')
        port_version = details.get('port-version', 0)
        ports_info.append((port, baseline, port_version))

    return ports_info

def check_for_updated_ports():
    updated_ports = []
    for port in get_vcpkg_ports():
        name, version, port_version = get_port_info(port)
        for port, baseline, baseline_version in parse_ports('3rdParty/linux/vcpkg/versions/baseline.json'):
            if name == port and (version != baseline or port_version != baseline_version):
                updated_ports.append((port, baseline, baseline_version))

    return updated_ports

updated_ports = check_for_updated_ports()
if (updated_ports):
    print('The following ports have been updated:')
    for port, baseline, baseline_version in updated_ports:
        print(f'{port}: {baseline}:{baseline_version}')
    sys.exit(1)
else:
    print('All ports are up-to-date.')
    sys.exit(0)



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import sys
import json

def main():
    if len(sys.argv) != 5:
        print("Usage: create_signing_metadata.py <output_file> <endpoint> <code_signing_account_name> <certificate_profile_name>")
        sys.exit(1)

    output_file = sys.argv[1]
    metadata = {
        "Endpoint": sys.argv[2],
        "CodeSigningAccountName": sys.argv[3],
        "CertificateProfileName": sys.argv[4]
    }

    with open(output_file, 'w', encoding='utf-8') as f:
        json.dump(metadata, f, ensure_ascii=False, indent=4)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2026 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

"""
Download BOINC installer for Windows based on platform and version type.

Usage:
    python3 download_boinc_installer.py <platform> <type>

    platform: x64 or arm64
    type: release or alpha

Examples:
    python3 download_boinc_installer.py x64 release
    python3 download_boinc_installer.py arm64 alpha
"""

import sys
import urllib.request
import xml.etree.ElementTree as ET


def download_file(url, filename):
    """Download a file from URL and save it with the given filename."""
    print(f"Downloading {url}...")
    urllib.request.urlretrieve(url, filename)
    print(f"Successfully downloaded to {filename}")


def get_download_url(xml_content, platform, version_type):
    """
    Parse XML and extract download URL based on platform and version type.

    Args:
        xml_content: XML content as string
        platform: 'x64' or 'arm64'
        version_type: 'release' or 'alpha'

    Returns:
        URL string or None if not found
    """
    root = ET.fromstring(xml_content)

    # Map input parameters to XML values
    platform_map = {
        'x64': 'Windows Intel-compatible 64-bit',
        'arm64': 'Windows ARM 64-bit'
    }

    version_map = {
        'release': 'Recommended version',
        'alpha': 'Development version'
    }

    target_platform = platform_map.get(platform)
    target_version = version_map.get(version_type)

    if not target_platform or not target_version:
        print(f"Error: Invalid platform '{platform}' or type '{version_type}'")
        return None

    # Find matching version entry
    for version in root.findall('version'):
        platform_elem = version.find('platform')
        description_elem = version.find('description')
        url_elem = version.find('url')

        if (platform_elem is not None and platform_elem.text == target_platform and
            description_elem is not None and description_elem.text == target_version and
            url_elem is not None):
            return url_elem.text

    return None


def main():
    if len(sys.argv) != 3:
        print("Usage: python3 download_boinc_installer.py <platform> <type>")
        print("  platform: x64 or arm64")
        print("  type: release or alpha")
        sys.exit(1)

    platform = sys.argv[1].lower()
    version_type = sys.argv[2].lower()

    if platform not in ['x64', 'arm64']:
        print(f"Error: Invalid platform '{platform}'. Use 'x64' or 'arm64'")
        sys.exit(1)

    if version_type not in ['release', 'alpha']:
        print(f"Error: Invalid type '{version_type}'. Use 'release' or 'alpha'")
        sys.exit(1)

    xml_url = 'https://boinc.berkeley.edu/download_all.php?xml=1'

    print(f"Fetching version information from {xml_url}...")
    try:
        with urllib.request.urlopen(xml_url) as response:
            xml_content = response.read().decode('utf-8')
    except Exception as e:
        print(f"Error downloading XML: {e}")
        sys.exit(1)

    download_url = get_download_url(xml_content, platform, version_type)

    if not download_url:
        if version_type == 'alpha':
            print(f"Development version for {platform} is not available. This is normal.")
            sys.exit(0)
        else:
            print(f"Error: Could not find download URL for {platform} {version_type}")
            sys.exit(1)

    output_filename = f"{platform}_{version_type}.exe"

    try:
        download_file(download_url, output_filename)
        print(f"Download completed successfully: {output_filename}")
    except Exception as e:
        print(f"Error downloading file: {e}")
        sys.exit(1)


if __name__ == '__main__':
    main()
